
# This code analyzes real patient data and
# reproduces Table 7, Table 8, Figure 4a and Figure 4b of 
# Hypothesis Testing Methods for Multivariate Multi-Occasion Intra-Individual Change

source("estimate_theta.R")
source("hypothesis_test_omnibus.R")

library(ggplot2)
library(reshape2)
library(dplyr)

# prior distribution for theta estimation
n_dims <- 3
D <- 1.7
max_len <- 60
prior_mu <- rep(0, n_dims)
inv_prior_cov <- solve(matrix(10 * c(1, 0.6,  0.46, 0.6, 1, 0.83, 0.46, 0.83, 1), nrow = 3))

# read item parameters and patient records
item_bank <- as.matrix(read.csv("item_bank.csv", header = TRUE, na.strings = "NA"))[, -1]
test_records <- read.csv("patient_data.csv")

patient_idx <- unique(test_records$patient_id)
n_patients <- length(patient_idx)

test_names <- c("LRT", "MWT", "ST", "MMWT")
col_names <- c(
  "patient_id", "n_occasions", 
  paste0(test_names, "_statistic"), paste0(test_names, "_decision"),
  paste0(rep(paste0("theta", 1:3), 4), "_occasion", rep(1:4, each = 3))
)
output <- data.frame(matrix(nrow = n_patients, ncol = length(col_names), dimnames = list(c(), col_names)))

for (i in 1:n_patients) {
  patient_id <- patient_idx[i]
  x <- test_records[test_records$patient_id == patient_id,]
  n_occasions <- nrow(x)
  n_items <- x[, "n_items"]
  total_n_items <- sum(n_items)
  
  # prepare data for performing omnibus tests
  item_params <- array(dim = c(total_n_items, 6))
  resp <- array(dim = total_n_items)
  theta_occasion <- as.matrix(x[, paste0("theta", 1:n_dims)])
  tmp <- c(0, cumsum(n_items))
  for (t in 1:n_occasions) {
    idx <- (tmp[t] + 1):(tmp[t + 1])
    item_names <- unlist(x[t, paste0("item", 1:n_items[t])])
    item_params[idx, ] <- item_bank[item_names, ]
    resp[idx] <- unlist(x[t, paste0("resp", 1:n_items[t])])
  }
  theta_pooled <- estimate_theta(item_params, resp, n_dims, D, "MAP", prior_mu, inv_prior_cov)$theta_hat
  
  # perform the tests and save the results in a dataframe
  output[i, 1] <- patient_id
  output[i, 2] <- n_occasions
  output[i, 3:4] <- LRT_omnibus(item_params, resp, theta_pooled, theta_occasion, n_dims, n_items)[c("test_statistic", "decision")]
  output[i, 5:6] <- MWT_omnibus(item_params, resp, theta_pooled, theta_occasion, n_dims, n_items)[c("test_statistic", "decision")]
  if (n_occasions == 4) {
    output[i, 7:8] <- MMWT_omnibus(item_params, resp, theta_pooled, theta_occasion, n_dims, n_items)[c("test_statistic", "decision")]
  } else {
    output[i, 7:8] <- NA
  }
  output[i, 9:10] <- ST_omnibus(item_params, resp, theta_pooled, n_dims, n_items)[c("test_statistic", "decision")]
  output[i, 11:22] <- c(as.vector(t(theta_occasion)), rep(NA, (4 - n_occasions) * 3))
}

# Table 7: percentage of examinees with psychometrically significant change
sig_change_percent <- matrix(nrow = 3, ncol = 4, dimnames = list(2:4, test_names))
for (i in 2:4) {
  tmp <- output[output$n_occasions == i, c("LRT_decision", "MWT_decision", "ST_decision", "MMWT_decision")]
  sig_change_percent[i - 1, ] <- round(colMeans(tmp), 4)
}

# Table 8: agreement between the omnibust tests
agreement <- matrix(nrow = 4, ncol = 4, dimnames = list(test_names, test_names))
for (i in 2:4) {
  cat(paste(i, "occasions \n"))
  tmp <- output[output$n_occasions == i, c("LRT_decision", "MWT_decision", "ST_decision", "MMWT_decision")]
  agreement <- round(cor(tmp), 2)
  print(agreement)
  cat("\n")
}

# Figure 4a: change patterns for 10 patients with the largest LRT change index
tmp <- output %>% filter(n_occasions == 4) %>% arrange(desc(LRT_statistic)) %>% slice(1:10)
output_long <- melt(tmp[, c(1, 11:22)], id.vars = "patient_id", variable.name = "dim_time", value.name = "theta")
output_long$patient_id <- sapply(output_long$patient_id, function(x) paste("Patient", x))
output_long$dim_time <- as.character(output_long$dim_time)
output_long$dim <- sapply(output_long$dim_time, function(x) strsplit(x, "_occasion")[[1]][1])
output_long$dim <- sapply(output_long$dim, function(x) strsplit(x, "theta")[[1]][2])
output_long$time <- sapply(output_long$dim_time, function(x) as.numeric(strsplit(x, "_occasion")[[1]][2]))
output_long$time <- factor(output_long$time)
ggplot(output_long, aes(x = dim, y = theta)) +
  geom_line(aes(linetype = time, group = time)) +
  geom_point() +
  scale_linetype_manual(values = c("solid", "dashed", "dotdash", "dotted")) +
  facet_wrap(~ patient_id, ncol = 5) +
  labs(x = "Scale", y = expression(hat(theta)), linetype = "Occasion") +
  theme_bw(base_size = 14)

# Figure 4b: change patterns for 10 patients with the smallest LRT change index
tmp <- output %>% filter(n_occasions == 4) %>% arrange(LRT_statistic) %>% slice(1:10)
output_long <- melt(tmp[, c(1, 11:22)], id.vars = "patient_id", variable.name = "dim_time", value.name = "theta")
output_long$patient_id <- sapply(output_long$patient_id, function(x) paste("Patient", x))
output_long$dim_time <- as.character(output_long$dim_time)
output_long$dim <- sapply(output_long$dim_time, function(x) strsplit(x, "_occasion")[[1]][1])
output_long$dim <- sapply(output_long$dim, function(x) strsplit(x, "theta")[[1]][2])
output_long$time <- sapply(output_long$dim_time, function(x) as.numeric(strsplit(x, "_occasion")[[1]][2]))
output_long$time <- factor(output_long$time)
ggplot(output_long, aes(x = dim, y = theta)) +
  geom_line(aes(linetype = time, group = time)) +
  geom_point() +
  scale_linetype_manual(values = c("solid", "dashed", "dotdash", "dotted")) +
  facet_wrap(~ patient_id, ncol = 5) +
  labs(x = "Scale", y = expression(hat(theta)), linetype = "Occasion") +
  theme_bw(base_size = 14)

