source("log_lik.R")

estimate_theta <- function(item_params, resp, n_dims, D, method = "MLE", prior_mu = NULL, inv_prior_cov = NULL) {
  
  ##############################################################################
  # estimate theta using MLE
  ##############################################################################
  # Parameters
  # item_params: a matrix of item parameters (only for items that are administered)
  # resp: a vector of responses
  # n_dims: no. of latent trait dimensions
  # D: scaling constant, 1 for logistic model, 1.7 for normal ogive model
  # method: theta estimation method (MLE or MAP)
  # prior_mu: mean of the prior distribution for theta
  # inv_prior_cov: inverse of the covariance matrix of the prior distribution for theta
  #
  # Returns
  # a list including
  # theta_hat: a vector of theta estimates
  # std_error: a vector of standard errors
  # det_fi: the determinant of the observed fisher information matrix
  ##############################################################################
  
  f <- function(theta) {
    if (method == "MLE") prior <- 0
    else if (method == "MAP") prior <- -0.5 * crossprod(theta - prior_mu, inv_prior_cov) %*% (theta - prior_mu)
    temp <- prior + log_lik(item_params, resp, n_dims, D, theta)
    return(-temp)
  }
  
  est <- optim(rep(0, n_dims), f,
               lower = rep(-4, n_dims), upper = rep(4, n_dims), 
               method = "L-BFGS-B", hessian = TRUE)
  
  theta_hat <- est$par
  std_error <- sqrt(diag(solve(est$hessian)))
  det_fi <- det(est$hessian)
  return(list(theta_hat = theta_hat, std_error = std_error, det_fi = det_fi))
}