log_lik <- function(item_params, resp, n_dims, D, theta, order = 0) {
  
  ##############################################################################
  # compute the (1st/2nd derivatives of) log likelihood
  ##############################################################################
  # Parameters
  # item_params: a matrix of item parameters (only for items that are administered)
  # resp: a vector of responses
  # n_dims: no. of latent trait dimensions
  # D: scaling constant, 1 for logistic model, 1.7 for normal ogive model
  # theta: current theta estimates
  # order: order of derviative, takes value 0, 1, 2
  #
  # Returns
  # a scalar/vector/matrix representing the 0th/1st/2nd derivatives of log likelihood
  ##############################################################################
  
  a <- item_params[, 1:n_dims, drop = F]
  b <- item_params[, (n_dims + 1):ncol(item_params), drop = F]
  n <- nrow(item_params) # no. of items
  
  # do some manipulation to vectorize the computation of log likelihood
  # when b = -1e8, cum_P = 1; when b = 1e8, cum_P = 0
  b[is.na(b)] <- 1e8
  b <- cbind(-1e8, b, 1e8)
  
  # store which boundary parameter to extract from each item
  id1 <- cbind(1:n, resp)
  id2 <- cbind(1:n, resp + 1)
  
  # cumulative probabilities
  cum_P1 <- 1 / (1 + exp(-D * rowSums(a * t(outer(theta, b[id1], "-")))))
  cum_P2 <- 1 / (1 + exp(-D * rowSums(a * t(outer(theta, b[id2], "-")))))
  
  # marginal probabilities
  P <- cum_P1 - cum_P2
  
  if (order == 0) {
    # log likelihood
    L <- sum(log(cum_P1 - cum_P2))
    return(L)
  } else if (order >= 1) {
    # first derivatives of marginal probabilities
    d_P <- sapply(1:n_dims, function(n_dims) D * a[, n_dims] * (cum_P1 * (1 - cum_P1) - cum_P2 * (1 - cum_P2)))
    # first derivatives of log likelihood
    d_L <- sapply(1:n_dims, function(n_dims) sum(d_P[, n_dims] / P))
    if (order == 2) {
      # second derivatives of log likelihood
      d2_L <- array(0, c(n_dims, n_dims))
      for (i in 1:n_dims) {
        for (j in 1:n_dims) {
          if (j >= i) { # only need to compute upper triangular entries
            # second derivatives of marginal probabilities
            d2_P <- D ^ 2 * a[, i] * a[, j] * (cum_P1 * (1 - cum_P1) * (1 - 2 * cum_P1) - cum_P2 * (1 - cum_P2) * (1 - 2 * cum_P2))
            d2_L[i, j] <- sum(d2_P / P - d_P[, i] * d_P[, j] / P ^ 2)
          } else
            d2_L[i, j] <- d2_L[j, i] # because the matrix is symmetric
        }
      }
      return(d2_L)
    }
    return(d_L)
  }
}
